/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the graphics subsystem.
*
*   This template is responsible to access the graphics subsystem within an
*   embedded Linux environment. The display is accessed by using EGL via the
*   graphics subsystem (e.g. Wayland, DRM, X11 or fbdev).
*
*   Important: This file is intended to be used as a template. Please adapt the
*   implementation and declarations according your particular hardware.
*
*******************************************************************************/

#include "ewconfig.h"
#include <GLES2/gl2.h>
#include <EGL/egl.h>
#include <EGL/eglext.h>

#include <string.h>
#include <fcntl.h>
#include "gfx_system.h"
void EwPrint( const char* aFormat, ... );

static EGLDisplay        EglDisplay        = 0;
static EGLSurface        EglSurface        = 0;
static EGLContext        EglContext        = 0;
static GLint             FrameBuffer       = 0;
static int               SurfaceWidth      = 0;
static int               SurfaceHeight     = 0;


/*******************************************************************************
* FUNCTION:
*   GfxSystemInit
*
* DESCRIPTION:
*   The function GfxSystemInit is responsible to get an access to the
*   graphics subsystem. In case that the underlying graphics subsystem contains
*   a window manager, the given size is used to create a suitable window.
*   Otherwise, the given size is ignored, because the size of the frame buffer
*   cannot be changed.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int GfxSystemInit( void )
{
  const char *          eglVendor;
  EGLConfig             eglConfig           = 0;
  EGLNativeDisplayType  nativeDisplayType   = EGL_DEFAULT_DISPLAY;
  EGLNativeWindowType   eglNativeWindowType = 0;
  EGLint                num_configs;
  EGLint                contextAttribs[]  =
  {
    EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE
  };
  EGLint                configAttribs[] =
  {
    EGL_SURFACE_TYPE,      EGL_WINDOW_BIT,
    EGL_RENDERABLE_TYPE,   EGL_OPENGL_ES2_BIT,
    EGL_BUFFER_SIZE,       32,
    EGL_COLOR_BUFFER_TYPE, EGL_RGB_BUFFER,
    EGL_RED_SIZE,          8,
    EGL_GREEN_SIZE,        8,
    EGL_BLUE_SIZE,         8,
    EGL_ALPHA_SIZE,        8,
    EGL_NATIVE_RENDERABLE, EGL_TRUE,
    EGL_NONE
  };

  /* initalize EGL sub-system */
  #ifdef EGL_API_FB
    nativeDisplayType = fbGetDisplayByIndex( 0 );
  #endif

  EglDisplay = eglGetDisplay( nativeDisplayType );
  eglInitialize( EglDisplay, 0, 0 );
  eglBindAPI( EGL_OPENGL_ES_API );
  eglChooseConfig( EglDisplay, configAttribs, &eglConfig, 1, &num_configs );

  if ( num_configs != 1 )
    return 0;

  eglVendor = eglQueryString( EglDisplay, EGL_VENDOR );
  if ( strstr( eglVendor, "AMD" ))
    eglNativeWindowType = ( EGLNativeWindowType ) open( "/dev/fb0", O_RDWR );
  #if (defined EGL_API_FB)
  else if ( strstr( eglVendor, "Vivante" ))
  {
    eglNativeWindowType = fbCreateWindow( nativeDisplayType, 0, 0, 0, 0 );
  }
  #endif

  EglSurface = eglCreateWindowSurface( EglDisplay, eglConfig, eglNativeWindowType, 0 );
  EglContext = eglCreateContext( EglDisplay, eglConfig, EGL_NO_CONTEXT, contextAttribs );

  #if EW_PERFORM_FULLSCREEN_UPDATE == 0
    EGLBoolean  eglResult;
    eglResult = eglSurfaceAttrib( EglDisplay, EglSurface, EGL_SWAP_BEHAVIOR, EGL_BUFFER_PRESERVED );
    if ( eglResult != EGL_TRUE )
      EwPrint( "\nWARNING: egl swap behaviour - buffer preservation not supported (egl error: 0x%04X)\n" \
        "Use macro EW_PERFORM_FULLSCREEN_UPDATE = 1\n\n", eglGetError());
  #else
    eglSurfaceAttrib( EglDisplay, EglSurface, EGL_SWAP_BEHAVIOR, EGL_BUFFER_DESTROYED );
  #endif

  eglMakeCurrent( EglDisplay, EglSurface, EglSurface, EglContext );

  /* set swap interval to display framerate */
  eglSwapInterval( EglDisplay, 1 );

  /* get the framebuffer and its size in pixel */
  glGetIntegerv( GL_FRAMEBUFFER_BINDING, &FrameBuffer);
  eglQuerySurface( EglDisplay, EglSurface, EGL_WIDTH, &SurfaceWidth );
  eglQuerySurface( EglDisplay, EglSurface, EGL_HEIGHT, &SurfaceHeight );

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemDone
*
* DESCRIPTION:
*   The function GfxSystemDone is responsible to close the access to the
*   graphics subsystem.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void GfxSystemDone( void )
{
  /* shutdown the EGL sub-system */
  eglMakeCurrent( EglDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT ) ;
  eglDestroySurface( EglDisplay, EglSurface );
  eglDestroyContext( EglDisplay, EglContext);
  eglTerminate( EglDisplay );
  eglReleaseThread();
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemProcess
*
* DESCRIPTION:
*   The function GfxSystemProcess drives the graphics subsystem or window
*   manager in order to process all pending events.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   1, if further processing is needed, 0 otherwise.
*
*******************************************************************************/
int GfxSystemProcess( void )
{
  return 1;
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemCreateSurface
*
* DESCRIPTION:
*   The function GfxSystemCreateSurface creates a graphic surface with the
*   given dimensions.
*
* ARGUMENTS:
*   aWidth  - Required surface width.
*   aHeight - Required surface height.
*
* RETURN VALUE:
*   Returns 1, if native window could be created, otherwise 0.
*
*******************************************************************************/
int GfxSystemCreateSurface( int aWidth, int aHeight )
{
   return 0;
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemUpdateSurface
*
* DESCRIPTION:
*   The function GfxSystemUpdateSurface updates the graphic surface.
*
* ARGUMENTS:
*   aSurfaceContext  - Surface context
*   aX1              - Surface update area left.
*   aY1              - Surface update area topt.
*   aX2              - Surface update area right.
*   aY2              - Surface update area bottom.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/
void GfxSystemUpdateSurface ( void * aSurfaceContext, int aX1, int aY1, int aX2, int aY2 )
{
  /* perform the swap if there was something drawn on the screen */
  eglSwapBuffers( EglDisplay, EglSurface );
}


/*******************************************************************************
* FUNCTION:
*   GfxSystemGetInfo
*
* DESCRIPTION:
*   The function GfxSystemGetInfo returns the graphic system info struct.
*
* ARGUMENTS:
*   aGfxSystemInfo  - Pointer to return the graphic system info struct.
*
* RETURN VALUE:
*   None.
*
*******************************************************************************/
void GfxSystemGetInfo( XGfxSystemInfo* aGfxSystemInfo )
{
  if ( ! aGfxSystemInfo )
    return;

  memset( aGfxSystemInfo, 0, sizeof( *aGfxSystemInfo ));
  aGfxSystemInfo->Type        = GFX_SYSTEM_FBDEV;
  aGfxSystemInfo->FrameBuffer = (void*)(intptr_t)FrameBuffer;
  aGfxSystemInfo->Display     = EglDisplay;
  aGfxSystemInfo->Surface     = EglSurface;
  aGfxSystemInfo->Context     = EglContext;
  aGfxSystemInfo->Width       = SurfaceWidth;
  aGfxSystemInfo->Height      = SurfaceHeight;
}


/* mli, msy */
